/*--------------------------------------------------
Telnet to Serial AccessPoint for ESP8266 
for ESP8266 adapted Arduino IDE

by Stefan Thesen 08/2015 - free for anyone
http://blog.thesen.eu

Creates an accesspoint or network client which can 
be connected by telnet; e.g. telnet 192.168.4.1
Telnet input is sent to serial and vice versa.

Serial output can e.g. be used to steer an attached
Arduino or other serial interfaces.
Please take care for levels of the serial lines.

Code inspired by a post of ghost on github:
https://github.com/esp8266/Arduino/issues/307
--------------------------------------------------*/
#include "Arduino.h"
#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include "WiFiManager.h"
#include <SoftwareSerial.h>


#define MYPORT_TX 1
#define MYPORT_RX 3

EspSoftwareSerial::UART myPort;
////////////////////////////////////
// settings for Telnet2Serial Bridge
////////////////////////////////////

// max number of clients that can connect
#define MAX_NO_CLIENTS 5
const WiFiMode wifi_mode = WIFI_STA;     // set WIFI_AP for access-point or WIFI_STA for WIFI client

const int iSerialSpeed = 9600;          // speed of the serial connection
const bool bSuppressLocalEcho = false;   // shall local echo in telnet be suppressed (usually yes)

WiFiManager wifiManager;

// Create an instance of the server on Port 23
WiFiServer server(23);
WiFiClient pClientList[MAX_NO_CLIENTS]; 

uint8_t buffer[1024];

void setup() 
{
  // start serial
  //Serial.begin(iSerialSpeed);
  myPort.begin(9600, SWSERIAL_8N1, MYPORT_RX, MYPORT_TX, false);
  pinMode(3, INPUT);
  digitalWrite(3, LOW); // disable internal pullup

  wifiManager.autoConnect("VC870");
  wifiManager.setConfigPortalTimeout(180);

  wifiManager.setHostname("VC870");

  ArduinoOTA.onStart([]() {
    myPort.println("Start");
  });
  ArduinoOTA.onEnd([]() {
    myPort.println("\nEnd");
  });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
    myPort.printf("Progress: %u%%\r", (progress / (total / 100)));
  });
  ArduinoOTA.onError([](ota_error_t error) {
    myPort.printf("Error[%u]: ", error);
    if (error == OTA_AUTH_ERROR) myPort.println("Auth Failed");
    else if (error == OTA_BEGIN_ERROR) myPort.println("Begin Failed");
    else if (error == OTA_CONNECT_ERROR) myPort.println("Connect Failed");
    else if (error == OTA_RECEIVE_ERROR) myPort.println("Receive Failed");
    else if (error == OTA_END_ERROR) myPort.println("End Failed");
  });
  ArduinoOTA.begin();
  myPort.println("Ready");
  myPort.print("IP address: ");
  myPort.println(WiFi.localIP());
  server.begin();
  server.setNoDelay(true);

}



void loop() 
{ 
  int ii;

  /////////////////////
  // handle new clients
  /////////////////////
  if (server.hasClient())
  {
    bool bFoundPlace=false;
    
    // search a free spot
    for(ii = 0; ii < MAX_NO_CLIENTS; ii++)
    {
      if (!pClientList[ii] || !pClientList[ii].connected())
      {
        // remove old connections
        if(pClientList[ii]) 
        {
          pClientList[ii].stop();
        }

        // new client
        pClientList[ii] = server.accept();
        if (bSuppressLocalEcho) 
        { 
          pClientList[ii].write("\xFF\xFB\x01", 3); 
        }
        bFoundPlace=true;
        break;
      }
    }

    //no free spot --> sorry
    if (!bFoundPlace)
    {
      WiFiClient client = server.accept();
      client.stop();
    }
  }



  /////////////////////
  // Serial --> Telnet
  /////////////////////
  memset(buffer, 0, sizeof(buffer));
  int length = myPort.available();
  if(length > 0)
  {

    length = myPort.readBytesUntil('\n', buffer, sizeof(buffer)-1);
    buffer[length] = '\n';
    length++;
    if(length >0){
      for(ii = 0; ii < MAX_NO_CLIENTS; ii++)
      {
        if (pClientList[ii] && pClientList[ii].connected())
        {
          pClientList[ii].write(buffer, length);
          
      
        }
      }
    }

    
  }
  ArduinoOTA.handle();
}

